from django.utils.translation import ugettext as _

DIRECTIONALITY_CHOICES = (
    ('Inbound', _('Inbound')),
    ('Outbound', _('Outbound')),
)

API_KEY = 'prtl6749387986743898559646983194'
CURRENCY = 'EUR'
MARKET = 'ES'
LOCALE = 'en-GB'


# coding=utf-8
from django.contrib import admin

from skyscannerSDK.models import Place, Carrier, Leg, FlightSearch, Agent, Segment, PricingOption, Itinerary


class PricingOptionInline(admin.StackedInline):
    model = PricingOption
    extra = 0
    filter_horizontal = ('agents',)


class ItineraryInline(admin.StackedInline):
    model = Itinerary
    extra = 0


class PlaceAdmin(admin.ModelAdmin):
    list_display = ('id', 'code', 'name', 'type')


class CarrierAdmin(admin.ModelAdmin):
    list_display = ('id', 'display_code', 'name')


class LegAdmin(admin.ModelAdmin):
    list_display = ('id', 'departure_place', 'arrival_place', 'departure', 'arrival', 'count_stops', 'flight_duration')
    filter_horizontal = ('segments', 'carriers', 'operating_carriers', 'stops')
    search_fields = ('departure_place__name', 'arrival_place__name')

    def count_stops(self, obj):
        return obj.stops.count()

    def flight_duration(self, obj):
        return str(obj.duration/60) + "h " + str(obj.duration%60) + "min"


class SegmentAdmin(admin.ModelAdmin):
    list_display = ('id', 'departure_place', 'arrival_place', 'departure', 'arrival', 'carrier', 'flight_number', 'flight_duration')

    def flight_duration(self, obj):
        return str(obj.duration / 60) + "h " + str(obj.duration % 60) + "min"


class ItineraryAdmin(admin.ModelAdmin):
    list_display = ('id', 'inbound_leg', 'outbound_leg', 'min_price')
    inlines = (PricingOptionInline,)

    def min_price(self, obj):
        return str(obj.pricingoption_set.all().first().price) + "€"


class FlightSearchAdmin(admin.ModelAdmin):
    list_display = ('id', 'origin', 'destination', 'outbound', 'inbound', 'created')
    inlines = (ItineraryInline,)
    date_hierarchy = 'created'
    list_filter = ('origin', 'destination')


class AgentAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'type', 'optimised_for_mobile')
    list_filter = ('type', 'optimised_for_mobile')


admin.site.register(Place, PlaceAdmin)
admin.site.register(Carrier, CarrierAdmin)
admin.site.register(Leg, LegAdmin)
admin.site.register(Segment, SegmentAdmin)
admin.site.register(Itinerary, ItineraryAdmin)
admin.site.register(FlightSearch, FlightSearchAdmin)
admin.site.register(Agent, AgentAdmin)


from django.utils.dateparse import parse_datetime
from pytz import UTC
from skyscanner.skyscanner import Flights

from constants import API_KEY, MARKET, CURRENCY, LOCALE
from skyscannerSDK.models import Place, PlaceType, FlightSearch, Carrier, Agent, AgentType, Leg, JourneyMode, Segment, \
    Itinerary, PricingOption


def search_flights(origin, destination, outbound, inbound, passengers):
    flights_service = Flights(API_KEY)
    result = flights_service.get_result(
        country=MARKET,
        currency=CURRENCY,
        locale=LOCALE,
        originplace=origin,
        destinationplace=destination,
        outbounddate=outbound.strftime('%Y-%m-%d'),
        inbounddate=inbound.strftime('%Y-%m-%d'),
        adults=passengers)

    if result.status_code is 200:
        fs = FlightSearch.objects.create(
            origin=origin,
            destination=destination,
            outbound=outbound,
            inbound=inbound,
            passengers=passengers,
            status=result.parsed['Status'],
            query=result.parsed['Query'],
            session_key=result.parsed['SessionKey']
        )
        return format_flight_search(fs, result.parsed)
    else:
        raise SearchErrorException(result.status_code)


def format_flight_search(flight_search, result):

    update_places(result)
    update_carriers(result)
    update_agents(result)
    get_legs(result)
    get_itineraries(result, flight_search)

    return flight_search


def update_places(result):

    places = result['Places']

    for place in places:
        if not Place.objects.filter(pk=place['Id']):
            Place.objects.create(
               id = place['Id'],
               code = place['Code'],
               name = place['Name'],
               type = PlaceType.objects.get_or_create(name=place['Type'])[0],
               parentId = place.get('parentId', 0),
            )


def update_carriers(result):

    carriers = result['Carriers']

    for carrier in carriers:
        if not Carrier.objects.filter(pk=carrier['Id']):
            Carrier.objects.create(
                id=carrier['Id'],
                code=carrier['Code'],
                display_code=carrier['DisplayCode'],
                name=carrier['Name'],
                image=carrier['ImageUrl']
            )


def update_agents(result):

    agents = result['Agents']

    for agent in agents:
        if not Agent.objects.filter(pk=agent['Id']):
            Agent.objects.create(
                id = agent['Id'],
                image = agent['ImageUrl'],
                name = agent['Name'],
                type = AgentType.objects.get_or_create(name=agent['Type'])[0],
                optimised_for_mobile= agent['OptimisedForMobile'],
            )


def get_legs(result):

    legs = result['Legs']
    segments = result['Segments']

    for leg in legs:

        l, created = Leg.objects.get_or_create(
            id = leg['Id'],
            departure_place = Place.objects.get(pk=leg['OriginStation']),
            arrival_place = Place.objects.get(pk=leg['DestinationStation']),
            departure = UTC.localize(parse_datetime(leg['Departure']), is_dst=True),
            arrival =  UTC.localize(parse_datetime(leg['Arrival']), is_dst=True),
            duration = leg['Duration'],
            directionality = leg['Directionality'],
            journey_mode = JourneyMode.objects.get_or_create(name=leg['JourneyMode'])[0]
        )

        carriers = leg['Carriers']
        ocarriers = leg['OperatingCarriers']
        stops = leg['Stops']
        lsegments = leg['SegmentIds']

        for carrier in carriers:
            l.carriers.add(Carrier.objects.get(pk=carrier))

        for ocarrier in ocarriers:
            l.operating_carriers.add(Carrier.objects.get(pk=ocarrier))

        for stop in stops:
            if stop != 0:
                l.stops.add(Place.objects.get(pk=stop))

        for lsegment in lsegments:
            l.segments.add(update_segment(lsegment, segments))

        l.save()


def update_segment(lsegment, segments):
    s = find(lsegment, segments)

    segment, created=Segment.objects.get_or_create(
        departure_place=Place.objects.get(pk=s['OriginStation']),
        arrival_place=Place.objects.get(pk=s['DestinationStation']),
        departure=UTC.localize(parse_datetime(s['DepartureDateTime']), is_dst=True),
        arrival=UTC.localize(parse_datetime(s['ArrivalDateTime']), is_dst=True),
        duration=s['Duration'],
        directionality=s['Directionality'],
        journey_mode=JourneyMode.objects.get_or_create(name=s['JourneyMode'])[0],
        flight_number=s['FlightNumber'],
        carrier=Carrier.objects.get(pk=s['Carrier']),
        operating_carrier=Carrier.objects.get(pk=s['OperatingCarrier'])
    )

    return segment


def find(value, dict):
    for d in dict:
        if d.get('Id', None) == value:
            return d


def get_itineraries(result, flight_search):
    itineraries = result['Itineraries']

    for itinerary in itineraries:
        i = Itinerary.objects.create(
            flight_search=flight_search,
            inbound_leg=Leg.objects.get(id=itinerary['InboundLegId']),
            outbound_leg=Leg.objects.get(id=itinerary['OutboundLegId']),
            booking_details_link=itinerary['BookingDetailsLink']
        )

        pricing_options = itinerary['PricingOptions']

        for option in pricing_options:
            p = PricingOption.objects.create(
                itinerary=i,
                price=option['Price'],
                deeplink=option['DeeplinkUrl'],
                quote_age_in_min=option['QuoteAgeInMinutes']
            )

            agents = option['Agents']

            for agent in agents:
                p.agents.add(Agent.objects.get(pk=agent))

            p.save()


class SearchErrorException(Exception):
    pass


import uuid

from django.db import models
from django.db.models import Min, Avg
from django.utils.translation import ugettext as _
from django_extensions.db.models import TimeStampedModel

from constants import DIRECTIONALITY_CHOICES


class PlaceType(models.Model):
    class Meta:
        verbose_name = _('Place Type')
        verbose_name_plural = _('Place Types')

    name = models.CharField(max_length=200)

    def __unicode__(self):
        return self.name


class Place(models.Model):

    class Meta:
        verbose_name = _('Place')
        verbose_name_plural = _('Places')

    code = models.CharField(max_length=10)
    name = models.CharField(max_length=200)
    type = models.ForeignKey(PlaceType)
    parentId = models.IntegerField(default=1)

    def __unicode__(self):
        return self.code + ": " + self.name + ' (' + self.type.__unicode__() + ')'

    def get_city(self):
        if self.type.name == 'City':
            return self
        places=Place.objects.filter(name=self.name.split()[0], type__name='City')
        if places.count() == 1:
            return places.first()
        return Place.objects.filter(name=' '.join(self.name.split()[:2]), type__name='City').first()


class Carrier(models.Model):

    class Meta:
        verbose_name = _('Carrier')
        verbose_name_plural = _('Carriers')

    code = models.CharField(max_length=10)
    display_code = models.CharField(max_length=10)
    name = models.CharField(max_length=200)
    image = models.URLField()

    def __unicode__(self):
        return self.display_code + ": " + self.name


class JourneyMode(models.Model):
    class Meta:
        verbose_name = _('Journey Mode')
        verbose_name_plural = _('Journey Modes')

    name = models.CharField(max_length=200)

    def __unicode__(self):
        return self.name


class Segment(models.Model):

    class Meta:
        verbose_name = _('Segment')
        verbose_name_plural = _('Segments')

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    departure_place = models.ForeignKey(Place, related_name='segment_departure_place')
    arrival_place = models.ForeignKey(Place, related_name='segment_arrival_place')
    departure = models.DateTimeField()
    arrival = models.DateTimeField()
    carrier = models.ForeignKey(Carrier, related_name='carrier')
    operating_carrier = models.ForeignKey(Carrier, related_name='operating_carrier')
    flight_number = models.CharField(max_length=10)
    duration = models.IntegerField(default=0)
    directionality = models.CharField(max_length=30, default='Outbound', choices=DIRECTIONALITY_CHOICES)
    journey_mode = models.ForeignKey(JourneyMode)

    def __unicode__(self):
        return self.departure_place.code + " - " + self.arrival_place.code


class Leg(models.Model):

    class Meta:
        verbose_name = _('Leg')
        verbose_name_plural = _('Legs')

    id = models.SlugField(primary_key=True, editable=False)
    departure_place = models.ForeignKey(Place, related_name='departure_place')
    arrival_place = models.ForeignKey(Place, related_name='arrival_place')
    departure = models.DateTimeField()
    arrival = models.DateTimeField()
    carriers = models.ManyToManyField(Carrier, related_name='carriers')
    operating_carriers = models.ManyToManyField(Carrier, related_name='operating_carriers')
    stops = models.ManyToManyField(Place, related_name='stops')
    duration = models.IntegerField(default=0)
    directionality = models.CharField(max_length=30, default='Outbound', choices=DIRECTIONALITY_CHOICES)
    journey_mode = models.ForeignKey(JourneyMode)
    segments = models.ManyToManyField(Segment)

    def __unicode__(self):
        ucode = self.departure_place.code + " - " + self.arrival_place.code
        if self.stops.count() != 0:
            ucode += " (" + _("Via") + ":"
            for stop in self.stops.all():
                ucode += " " + stop.code
            ucode += ")"
        else:
            ucode += " (" + _("Direct") + ")"
        return ucode


class AgentType(models.Model):

    class Meta:
        verbose_name = _('Agent Type')
        verbose_name_plural = _('Agent Types')

    name = models.CharField(max_length=200)

    def __unicode__(self):
        return self.name


class Agent(models.Model):

    class Meta:
        verbose_name = _('Agent')
        verbose_name_plural = _('Agents')

    name = models.CharField(max_length=200)
    type = models.ForeignKey(AgentType)
    image = models.URLField()
    optimised_for_mobile = models.BooleanField(default=True)

    def __unicode__(self):
        return self.name


class FlightSearch(TimeStampedModel):

    class Meta:
        verbose_name = _('Flight Search')
        verbose_name_plural = _('Flight Searches')

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    origin = models.CharField(max_length=30)
    destination = models.CharField(max_length=30)
    outbound = models.DateField()
    inbound = models.DateField()
    passengers = models.IntegerField(default=1)
    status = models.CharField(max_length=200)
    query = models.TextField()
    session_key = models.CharField(max_length=200)

    def __unicode__(self):
        return self.origin + '-' + self.destination + ' (' + self.outbound.strftime('%Y-%m-%d') + "-" + self.inbound.strftime('%Y-%m-%d') + ')'


    def get_origin_city(self):
        return self.itinerary_set.first().outbound_leg.departure_place.get_city()


    def get_destination_city(self):
        return self.itinerary_set.first().outbound_leg.arrival_place.get_city()

    def get_min_price(self):
        return self.itinerary_set.all().annotate(min_price=Min('pricingoption__price')).order_by(
            'min_price').first()

    def get_max_price(self):
        return self.itinerary_set.all().annotate(min_price=Min('pricingoption__price')).order_by(
            '-min_price').first()

    def get_mean_price(self):
        return \
            self.itinerary_set.all().annotate(min_price=Min('pricingoption__price')).aggregate(Avg('min_price'))[
                'min_price__avg']


class Itinerary(TimeStampedModel):

    class Meta:
        verbose_name = _('Itinerary')
        verbose_name_plural = _('Itineraries')

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    flight_search = models.ForeignKey(FlightSearch)
    inbound_leg = models.ForeignKey(Leg, related_name='inbound')
    outbound_leg = models.ForeignKey(Leg, related_name='outbound')
    booking_details_link = models.TextField()


class PricingOption(models.Model):

    class Meta:
        verbose_name = _('Pricing Option')
        verbose_name_plural = _('Pricing Options')

    itinerary = models.ForeignKey(Itinerary)
    price = models.FloatField()
    agents = models.ManyToManyField(Agent)
    deeplink = models.URLField(max_length=2000)
    quote_age_in_min = models.IntegerField(default=1)

    def get_agent(self):
        return self.agents.all().first().name



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-30 22:30
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Agent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('image', models.URLField()),
                ('optimised_for_mobile', models.BooleanField(default=True)),
            ],
            options={
                'verbose_name': 'Agent',
                'verbose_name_plural': 'Agents',
            },
        ),
        migrations.CreateModel(
            name='AgentType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
            ],
            options={
                'verbose_name': 'Agent Type',
                'verbose_name_plural': 'Agent Types',
            },
        ),
        migrations.CreateModel(
            name='Carrier',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=10)),
                ('display_code', models.CharField(max_length=10)),
                ('name', models.CharField(max_length=200)),
                ('image', models.URLField()),
            ],
            options={
                'verbose_name': 'Carrier',
                'verbose_name_plural': 'Carriers',
            },
        ),
        migrations.CreateModel(
            name='FlightSearch',
            fields=[
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('origin', models.CharField(max_length=30)),
                ('destination', models.CharField(max_length=30)),
                ('outbound', models.DateField()),
                ('inbound', models.DateField()),
                ('passengers', models.IntegerField(default=1)),
                ('status', models.CharField(max_length=200)),
                ('query', models.TextField()),
                ('session_key', models.CharField(max_length=200)),
            ],
            options={
                'verbose_name': 'Flight Search',
                'verbose_name_plural': 'Flight Searches',
            },
        ),
        migrations.CreateModel(
            name='Itinerary',
            fields=[
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('booking_details_link', models.TextField()),
                ('flight_search', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='skyscannerSDK.FlightSearch')),
            ],
            options={
                'verbose_name': 'Itinerary',
                'verbose_name_plural': 'Itineraries',
            },
        ),
        migrations.CreateModel(
            name='JourneyMode',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
            ],
            options={
                'verbose_name': 'Journey Mode',
                'verbose_name_plural': 'Journey Modes',
            },
        ),
        migrations.CreateModel(
            name='Leg',
            fields=[
                ('id', models.SlugField(editable=False, primary_key=True, serialize=False)),
                ('departure', models.DateTimeField()),
                ('arrival', models.DateTimeField()),
                ('duration', models.IntegerField(default=0)),
                ('directionality', models.CharField(choices=[(b'Inbound', 'Inbound'), (b'Outbound', 'Outbound')], default=b'Outbound', max_length=30)),
            ],
            options={
                'verbose_name': 'Leg',
                'verbose_name_plural': 'Legs',
            },
        ),
        migrations.CreateModel(
            name='Place',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=10)),
                ('name', models.CharField(max_length=200)),
                ('parentId', models.IntegerField(default=1)),
            ],
            options={
                'verbose_name': 'Place',
                'verbose_name_plural': 'Places',
            },
        ),
        migrations.CreateModel(
            name='PlaceType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
            ],
            options={
                'verbose_name': 'Place Type',
                'verbose_name_plural': 'Place Types',
            },
        ),
        migrations.CreateModel(
            name='PricingOption',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('price', models.FloatField()),
                ('deeplink', models.URLField()),
                ('quote_age_in_min', models.IntegerField(default=1)),
                ('agents', models.ManyToManyField(to='skyscannerSDK.Agent')),
                ('itinerary', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='skyscannerSDK.Itinerary')),
            ],
            options={
                'verbose_name': 'Pricing Option',
                'verbose_name_plural': 'Pricing Options',
            },
        ),
        migrations.CreateModel(
            name='Segment',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('departure', models.DateTimeField()),
                ('arrival', models.DateTimeField()),
                ('flight_number', models.CharField(max_length=10)),
                ('duration', models.IntegerField(default=0)),
                ('directionality', models.CharField(choices=[(b'Inbound', 'Inbound'), (b'Outbound', 'Outbound')], default=b'Outbound', max_length=30)),
                ('arrival_place', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='segment_arrival_place', to='skyscannerSDK.Place')),
                ('carrier', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='carrier', to='skyscannerSDK.Carrier')),
                ('departure_place', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='segment_departure_place', to='skyscannerSDK.Place')),
                ('journey_mode', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='skyscannerSDK.JourneyMode')),
                ('operating_carrier', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='operating_carrier', to='skyscannerSDK.Carrier')),
            ],
            options={
                'verbose_name': 'Segment',
                'verbose_name_plural': 'Segments',
            },
        ),
        migrations.AddField(
            model_name='place',
            name='type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='skyscannerSDK.PlaceType'),
        ),
        migrations.AddField(
            model_name='leg',
            name='arrival_place',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='arrival_place', to='skyscannerSDK.Place'),
        ),
        migrations.AddField(
            model_name='leg',
            name='carriers',
            field=models.ManyToManyField(related_name='carriers', to='skyscannerSDK.Carrier'),
        ),
        migrations.AddField(
            model_name='leg',
            name='departure_place',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='departure_place', to='skyscannerSDK.Place'),
        ),
        migrations.AddField(
            model_name='leg',
            name='journey_mode',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='skyscannerSDK.JourneyMode'),
        ),
        migrations.AddField(
            model_name='leg',
            name='operating_carriers',
            field=models.ManyToManyField(related_name='operating_carriers', to='skyscannerSDK.Carrier'),
        ),
        migrations.AddField(
            model_name='leg',
            name='segments',
            field=models.ManyToManyField(to='skyscannerSDK.Segment'),
        ),
        migrations.AddField(
            model_name='leg',
            name='stops',
            field=models.ManyToManyField(related_name='stops', to='skyscannerSDK.Place'),
        ),
        migrations.AddField(
            model_name='itinerary',
            name='inbound_leg',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='inbound', to='skyscannerSDK.Leg'),
        ),
        migrations.AddField(
            model_name='itinerary',
            name='outbound_leg',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='outbound', to='skyscannerSDK.Leg'),
        ),
        migrations.AddField(
            model_name='agent',
            name='type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='skyscannerSDK.AgentType'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-08 12:24
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('skyscannerSDK', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pricingoption',
            name='deeplink',
            field=models.URLField(max_length=2000),
        ),
    ]
